#include <linux/version.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
#include <linux/device.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
# define usb_match_id(dev, intf, table) usb_match_id(intf, table)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
# define return_probe(priv_data, retval) return priv_data
# define usb_set_intfdata(intf, priv_data)
#else
# define return_probe(priv_data, retval) return retval
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 19)
#define BUG_ON(condition) do { if ((condition)!=0) BUG(); } while(0)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#define in_atomic in_interrupt
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
extern inline void module_put(struct module *mod)
{
	BUG_ON(mod != THIS_MODULE);
	MOD_DEC_USE_COUNT;
}
extern inline int try_module_get(struct module *mod)
{
	BUG_ON(mod != THIS_MODULE);
	MOD_INC_USE_COUNT;
	return 1;
}
#endif

#ifndef ALIGN
#define ALIGN(x,a) (((x)+(a)-1)&~((a)-1))
#endif

/* Workqueue / task queue backwards compatibility stuff */

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,41)
#include <linux/workqueue.h>
#else
#include <linux/tqueue.h>
#define work_struct tq_struct
#define INIT_WORK INIT_TQUEUE
#define schedule_work schedule_task
#endif

/* Interrupt handler backwards compatibility stuff */
#ifndef IRQ_NONE

#define IRQ_NONE
#define IRQ_HANDLED
typedef void irqreturn_t;

#endif

#ifdef __LINUX_USB_H
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
   /* Take note of the inline functions to hack around modversions */

# ifdef usb_submit_urb
   extern inline int usb_submit_urb_real(struct urb *urb)
   {
	return usb_submit_urb(urb);
   }
#  undef usb_submit_urb
#  define usb_submit_urb(urb, mem_flags) usb_submit_urb_real(urb)
# else
#  define usb_submit_urb(urb, mem_flags) usb_submit_urb(urb)
# endif

# ifdef usb_alloc_urb
   extern inline struct urb *usb_alloc_urb_real(int iso_packets)
   {
	return usb_alloc_urb(iso_packets);
   }
#  undef usb_alloc_urb
#  define usb_alloc_urb(iso_packets, mem_flags) usb_alloc_urb_real(iso_packets)
# else
#  define usb_alloc_urb(iso_packets, mem_flags) usb_alloc_urb(iso_packets)
# endif
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,69)
static inline
void usb_init_urb(struct urb *urb)
{
	memset(urb, 0, sizeof(*urb));
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0) /* tune me! */
	urb->count = (atomic_t)ATOMIC_INIT(1);
#endif
	spin_lock_init(&urb->lock);
}
#endif


#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#define bridge_bulk_in_callback(urb, pt_regs) \
		bridge_bulk_in_callback(urb)
#define bridge_request_out_callback(urb, pt_regs) \
		bridge_request_out_callback(urb)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#define URB_ASYNC_UNLINK USB_ASYNC_UNLINK
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
	
# define get_iface_desc(iface, alt_n) (&(iface)->altsetting[alt_n])

# define get_iface_ep_desc(iface, alt_n, n) \
		(&(iface)->altsetting[alt_n].endpoint[n])

#else

# define get_iface_desc(iface, alt_n) (&(iface)->altsetting[alt_n].desc)

# define get_iface_ep_desc(iface, alt_n, n) \
		(&(iface)->altsetting[alt_n].endpoint[n].desc)

#endif
#endif

#ifdef __LINUX_USB_H
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#include "firmware.h"
extern inline char *fw_dev_param(struct usb_device *udev, char *buf)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 20)
	snprintf(buf, FIRMWARE_NAME_MAX, "usb-%s-%s",
		 udev->bus->bus_name, udev->devpath);
#else
	snprintf(buf, FIRMWARE_NAME_MAX, "usb-%d-%d",
		 udev->bus->busnum, udev->devnum);
#endif
	return buf;
}
#else
#define fw_dev_param(udev, buf) (&udev->dev)
#endif
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
#define WARN_ON BUG_ON
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,20))
static inline void list_move(struct list_head *list, struct list_head *head)
{
        __list_del(list->prev, list->next);
        list_add(list, head);
}

static inline void list_move_tail(struct list_head *list,
				  struct list_head *head)
{
        __list_del(list->prev, list->next);
        list_add_tail(list, head);
}
#endif


#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
#ifdef _LINUX_NETDEVICE_H
#ifndef HAVE_FREE_NETDEV
extern inline void free_netdev(struct net_device *dev)
{
	kfree(dev);
}
#endif
#endif
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 22)
#include <linux/pci.h>
static inline char *pci_name(struct pci_dev *pdev)
{
	        return pdev->slot_name;
}
#endif


/* pcmcia_core module from pcmcia-cs doesn't export pcmcia_* functions
 * and requires using CardServices instead.  Taken from hostap driver. */
#ifdef __IN_PCMCIA_PACKAGE__
#undef pcmcia_access_configuration_register
#define pcmcia_access_configuration_register(handle, reg) \
	CardServices(AccessConfigurationRegister, handle, reg)

#undef pcmcia_register_client
#define pcmcia_register_client(handle, reg) \
	CardServices(RegisterClient, handle, reg)

#undef pcmcia_deregister_client
#define pcmcia_deregister_client(handle) \
	CardServices(DeregisterClient, handle)

#undef pcmcia_get_first_tuple
#define pcmcia_get_first_tuple(handle, tuple) \
	CardServices(GetFirstTuple, handle, tuple)

#undef pcmcia_get_next_tuple
#define pcmcia_get_next_tuple(handle, tuple) \
	CardServices(GetNextTuple, handle, tuple)

#undef pcmcia_get_tuple_data
#define pcmcia_get_tuple_data(handle, tuple) \
	CardServices(GetTupleData, handle, tuple)

#undef pcmcia_parse_tuple
#define pcmcia_parse_tuple(handle, tuple, parse) \
	CardServices(ParseTuple, handle, tuple, parse)

#undef pcmcia_get_configuration_info
#define pcmcia_get_configuration_info(handle, config) \
	CardServices(GetConfigurationInfo, handle, config)

#undef pcmcia_request_io
#define pcmcia_request_io(handle, req) \
	CardServices(RequestIO, handle, req)

#undef pcmcia_request_irq
#define pcmcia_request_irq(handle, req) \
	CardServices(RequestIRQ, handle, req)

#undef pcmcia_request_configuration
#define pcmcia_request_configuration(handle, req) \
	CardServices(RequestConfiguration, handle, req)

#undef pcmcia_release_configuration
#define pcmcia_release_configuration(handle) \
	CardServices(ReleaseConfiguration, handle)

#undef pcmcia_release_io
#define pcmcia_release_io(handle, req) \
	CardServices(ReleaseIO, handle, req)

#undef pcmcia_release_irq
#define pcmcia_release_irq(handle, req) \
	CardServices(ReleaseIRQ, handle, req)

#undef pcmcia_release_window
#define pcmcia_release_window(win) \
	CardServices(ReleaseWindow, win)

#undef pcmcia_get_card_services_info
#define pcmcia_get_card_services_info(info) \
	CardServices(GetCardServicesInfo, info)

#undef pcmcia_report_error
#define pcmcia_report_error(handle, err) \
	CardServices(ReportError, handle, err)

#undef pcmcia_reset_card
#define pcmcia_reset_card(handle, err) \
	CardServices(ReportError, handle, err)

#undef pcmcia_validate_cis
#define pcmcia_validate_cis(handle, reg) \
	CardServices(ValidateCIS, handle, reg)
#endif /* __IN_PCMCIA_PACKAGE__ */


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,70)
#ifdef _LINUX_DS_H

/* Icky old style pcmcia registration */

struct device_driver {
	char			name[DEV_NAME_LEN];
};

struct pcmcia_driver {
	dev_link_t		*(*attach)(void);
	void			(*detach)(dev_link_t *);
	struct module		*owner;
	struct device_driver	drv;
};

static int __init pcmcia_register_driver(struct pcmcia_driver *driver)
{
	servinfo_t serv;

	CardServices(GetCardServicesInfo, &serv);
	if (serv.Revision != CS_RELEASE_CODE) {
		printk(KERN_NOTICE "orinoco_cs: Card Services release "
		       "does not match!\n");
		return -1;
	}

	register_pccard_driver((dev_info_t *) &driver->drv.name,
			       driver->attach, driver->detach);
	return 0;
}


static void __exit pcmcia_unregister_driver(struct pcmcia_driver *driver)
{
	unregister_pccard_driver((dev_info_t *) &driver->drv.name);
}

#endif
#endif

extern inline void complete_all(struct completion *comp)
{
	/* Hopefully the real complete_all will soon be exported, in the mean
	 * while this should work. */
	complete(comp);
	complete(comp);
	complete(comp);
	complete(comp);
}

