/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: otg_hc_rh.c
 * Author: Biao Jia		1/21/02
 * Description:
 *    Root hub object handler.
 */
#include <os.h>
#include <stack_urb.h>
#include <otg.h>

char otg_hc_rh_StringDescriptor[] =
{
	66,
	0x03, /* String descriptor */
	0x00, 'T', /* total 42 bytes */
	0x00, 'r',
	0x00, 'a',
	0x00, 'n',
	0x00, 's',
	0x00, 'D',
	0x00, 'i',
	0x00, 'm',
	0x00, 'e',
	0x00, 'n',
	0x00, 's',
	0x00, 'i',
	0x00, 'o',
	0x00, 'n',
	0x00, ',',
	0x00, ' ',
	0x00, 'I',
	0x00, 'n',
	0x00, 'c',
	0x00, '.',
	0x00, 'O', /* offset 21 * 2,  total 14 bytes */
	0x00, 'T',
	0x00, 'G',
	0x00, ' ',
	0x00, '2',
	0x00, '4',
	0x00, '3',
	0x00, '0', /* offset 28 * 2,  total 8 bytes */
	0x00, '1',
	0x00, '0',
	0x00, '0',
};

void
otg_hc_rh_Init(otg_hc_rh_t *rh, otg_hc_t *hc)
{
	int i;

	rh->hc = hc;
	for(i = 0; i < otg_hc_rh_MaxPorts; i++)
	{
		otg_hc_rh_port_Init(&rh->ports[i], rh, i);
	}

	/* Initialize descriptors */
	rh->deviceDescriptor[0] = otg_hc_rh_DeviceDecriptorLength;
	rh->deviceDescriptor[1] = 0x01; /* Device descriptor */
	rh->deviceDescriptor[2] = otg_hc_rh_USBReleaseNo & 0xFF;
	rh->deviceDescriptor[3] = (otg_hc_rh_USBReleaseNo >> 8) & 0xFF;
	rh->deviceDescriptor[4] = 0x09; /* Hub class */
	rh->deviceDescriptor[5] = 0x00;
	rh->deviceDescriptor[6] = 0x00;
	rh->deviceDescriptor[7] = 0x08;
	rh->deviceDescriptor[8] = 0x20; /* TDI's vendor ID */
	rh->deviceDescriptor[9] = 0x0B;
	rh->deviceDescriptor[10] = 0x43; /* Product ID OTG 243 */
	rh->deviceDescriptor[11] = 0x02;
	rh->deviceDescriptor[12] = otg_hc_rh_ReleaseNo & 0xFF;
	rh->deviceDescriptor[13] = (otg_hc_rh_ReleaseNo >> 8) & 0xFF;
	rh->deviceDescriptor[14] = 0x00;
	rh->deviceDescriptor[15] = 42;
	rh->deviceDescriptor[16] = 56;
	rh->deviceDescriptor[17] = 0x01; /* only one configuration */

	/* Configuration descriptor */
	rh->configDescriptor[0] = otg_hc_rh_ConfigDecriptorLength;
	rh->configDescriptor[1] = 0x02; /* Configuration descriptor */
	i = otg_hc_rh_ConfigDecriptorLength + otg_hc_rh_InterfaceDecriptorLength +
		otg_hc_rh_EndpointDecriptorLength;
	rh->configDescriptor[2] = i & 0xFF; /* Total length */
	rh->configDescriptor[3] = (i >> 8) & 0xFF;
	rh->configDescriptor[4] = 0x01; /* Only one interface */
	rh->configDescriptor[5] = 0x01;
	rh->configDescriptor[6] = 0x00;
	rh->configDescriptor[7] = 0xC0; /* Self-powered */
	rh->configDescriptor[8] = 0x00;

	/* Interface descriptor */
	rh->interfaceDescriptor[0] = otg_hc_rh_InterfaceDecriptorLength;
	rh->interfaceDescriptor[1] = 0x04; /* Interface descriptor */
	rh->interfaceDescriptor[2] = 0x00;
	rh->interfaceDescriptor[3] = 0x00;
	rh->interfaceDescriptor[4] = 0x01; /* Number of endpoints */
	rh->interfaceDescriptor[5] = 0x09; /* Hub class */
	rh->interfaceDescriptor[6] = 0x00;
	rh->interfaceDescriptor[7] = 0x00;
	rh->interfaceDescriptor[8] = 0x00;

	/* Endpoint descriptor */
	rh->endpointDescriptor[0] = otg_hc_rh_EndpointDecriptorLength;
	rh->endpointDescriptor[1] = 0x05; /* Endpoint descriptor */
	rh->endpointDescriptor[2] = 0x81; /* IN pipe for status/change */
	rh->endpointDescriptor[3] = 0x03; /* Interrupt */
	rh->endpointDescriptor[4] = 0x08; /* Packet size 8 bytes */
	rh->endpointDescriptor[5] = 0x00;
	rh->endpointDescriptor[6] = 0xFF; /* 255ms interrupt interval */

	/* Hub Descriptor */
	rh->hubDescriptor[0] = otg_hc_rh_DecriptorLength;
	rh->hubDescriptor[1] = 0x29; /* Hub descriptor */
	rh->hubDescriptor[2] = otg_hc_rh_MaxPorts; /* Number of ports */
	rh->hubDescriptor[3] = 0x00;
	rh->hubDescriptor[4] = 0x00;
	rh->hubDescriptor[5] = 5; /* 10ms needed for port to power on */
	rh->hubDescriptor[6] = 0x00;
	rh->hubDescriptor[7] = 0xE0;
	rh->hubDescriptor[8] = 0x07;
}

void
otg_hc_rh_StartTimer(otg_hc_rh_t *rh)
{
	OS_DeclareTimer(*timer);

	timer = &rh->timer;
	OS_StartTimer(timer, otg_hc_rh_Timeout, rh, rh->interval);
}

void
otg_hc_rh_Timeout(unsigned long data)
{
	otg_hc_rh_t *rh = (otg_hc_rh_t *)data;

	if (rh->enableSend)
	{
		int i;
		UINT8 w;
		stack_urb_t *urb;

		/* Return status:
		 * bit0: 1 - hub status change detected
		 * bit1: 1 - port 1 status change detected
		 * bit1: 2 - port 2 status change detected
		 * bit1: 3 - port 3 status change detected
		 */
		w = 0;
		for(i = 0; i < rh->num_ports; i++)
		{
			if (otg_hc_rh_port_IsChanged(&rh->ports[i]))
			{
				w |= (1 << (i + 1));
			}
		}

		if (w)
		{
			/* If status change, send it back otherwise ignore (NAK) */
			urb = rh->urb;
			*((UINT8 *)stack_urb_GetBuffer(urb)) = w;
			stack_urb_SetActualLength(urb, 1);
			stack_urb_SetStatus(urb, 0);

			if (stack_urb_HasCallBack(urb))
			{
				stack_urb_CallBack(urb);
			}
		}
	}

	otg_hc_rh_StartTimer(rh);
}

int
otg_hc_rh_Submit(otg_hc_rh_t *rh, stack_urb_t *urb)
{
	int rc;

	if (stack_urb_IsIntrPipe(urb))
	{
		if (stack_urb_GetEndpoint(urb) != 1) /* IN EP = 1 */
		{
			return -EPIPE;
		}
		
		if (rh->enableSend)
		{
			return -ENXIO; /* Already sent */
		}

		rh->enableSend = 1;
		rh->urb = urb;
		rh->interval = stack_urb_GetIntrInterval(urb);

		otg_hc_rh_StartTimer(rh);

		return 0;
	}

	if (!stack_urb_IsCtrlPipe(urb) || stack_urb_GetEndpoint(urb))
	{
		return -EPIPE;
	}

	rc = otg_hc_rh_DoCommand(rh, urb);
	if (rc)
	{
		return rc;
	}

	return 0;
}

int
otg_hc_rh_DoCommand(otg_hc_rh_t *rh, stack_urb_t *urb)
{
	stack_setup_t *cmd;
	int type;
	int req;
	int wValue;
	int wIndex;
	int wLength;
	int rc;
	unsigned long flags;

	otg_LockChipIntr(otg_hc_GetOtg(rh->hc), flags);

	cmd = stack_urb_GetSetup(urb);
	type = stack_urb_GetReqestType(cmd);
	req = stack_urb_GetReqest(cmd);
	wValue = OS_Le16ToCpu(stack_urb_GetReqestValue(cmd));
	wIndex = OS_Le16ToCpu(stack_urb_GetReqestIndex(cmd));
	wLength = OS_Le16ToCpu(stack_urb_GetReqestLength(cmd));
#ifdef DEBUG_OTG_RH
info("otg_hc_rh_Docommand called with type=0x%x  req=0x%x wValue=0x%x",
	type, req, wValue);
info("otg_hc_rh_Docommand called with wIndex=0x%x  wLength=0x%x",
	wIndex, wLength);
#endif

	if (type & otg_hc_rh_ClassRequest)
	{
		/* Hub Class specific request */
		switch (req)
		{
			case otg_ep0_RequestGetStatus:
				rc = otg_hc_rh_GetHubStatus(rh, urb, type, wIndex);
				break;
			case otg_ep0_RequestClearFeature:
				rc = otg_hc_rh_ClearHubFeature(rh, urb, type, wValue, wIndex);
				break;
			case otg_ep0_RequestSetFeature:
				rc = otg_hc_rh_SetHubFeature(rh, urb, type, wValue, wIndex);
				break;
			case otg_ep0_RequestGetDescriptor:
				rc = otg_hc_rh_GetDescriptor(rh, urb, type, wValue, wLength);
				break;
			case otg_ep0_RequestSetDescriptor:
				rc = otg_hc_rh_SetDescriptor(rh, urb, type, wValue, wLength);
				break;
			default:
				rc = -EINVAL;
				break;
		}
	}
	else
	{
		/* Standard request */	
		switch (req)
		{
			case otg_ep0_RequestGetStatus:
				rc = otg_hc_rh_GetStatus(rh, urb, type, req);
				break;
			case otg_ep0_RequestClearFeature:
				rc = otg_hc_rh_ClearFeature(rh, type, wValue);
				break;
			case otg_ep0_RequestSetFeature:
				rc = otg_hc_rh_SetFeature(rh, type, wValue);
				break;
			case otg_ep0_RequestSetAddress:
				rc = otg_hc_rh_SetAddress(rh, type, wValue);
				break;
			case otg_ep0_RequestGetDescriptor:
				rc = otg_hc_rh_GetDescriptor(rh, urb, type, wValue, wLength);
				break;
			case otg_ep0_RequestSetDescriptor:
				rc = otg_hc_rh_SetDescriptor(rh, urb, type, wValue, wLength);
				break;
			case otg_ep0_RequestGetConfiguration:
				rc = otg_hc_rh_GetConfig(rh, urb, type, wValue);
				break;
			case otg_ep0_RequestSetConfiguration:
				rc = otg_hc_rh_SetConfig(rh, urb, type, wValue);
				break;
			default:
				rc = -EINVAL;
				break;
		}
	}

#ifdef DEBUG_OTG_RH
info("otg_hc_rh_DoCommand urb->status=%d", stack_urb_GetStatus(urb));
#endif
	stack_urb_SetStatus(urb, rc);
	stack_urb_SetDevice(urb, 0);
	otg_UnlockChipIntr(otg_hc_GetOtg(rh->hc), flags);

	if (stack_urb_HasCallBack(urb))
	{
		stack_urb_CallBack(urb);
	}

	return rc;
}
